"""Generated client library for securitycenter version v1beta2."""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.py import base_api
from googlecloudsdk.third_party.apis.securitycenter.v1beta2 import securitycenter_v1beta2_messages as messages


class SecuritycenterV1beta2(base_api.BaseApiClient):
  """Generated client library for service securitycenter version v1beta2."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://securitycenter.googleapis.com/'
  MTLS_BASE_URL = 'https://securitycenter.mtls.googleapis.com/'

  _PACKAGE = 'securitycenter'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1beta2'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'SecuritycenterV1beta2'
  _URL_VERSION = 'v1beta2'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new securitycenter handle."""
    url = url or self.BASE_URL
    super(SecuritycenterV1beta2, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.folders_containerThreatDetectionSettings = self.FoldersContainerThreatDetectionSettingsService(self)
    self.folders_eventThreatDetectionSettings = self.FoldersEventThreatDetectionSettingsService(self)
    self.folders_rapidVulnerabilityDetectionSettings = self.FoldersRapidVulnerabilityDetectionSettingsService(self)
    self.folders_securityHealthAnalyticsSettings = self.FoldersSecurityHealthAnalyticsSettingsService(self)
    self.folders_virtualMachineThreatDetectionSettings = self.FoldersVirtualMachineThreatDetectionSettingsService(self)
    self.folders_webSecurityScannerSettings = self.FoldersWebSecurityScannerSettingsService(self)
    self.folders = self.FoldersService(self)
    self.organizations_containerThreatDetectionSettings = self.OrganizationsContainerThreatDetectionSettingsService(self)
    self.organizations_eventThreatDetectionSettings = self.OrganizationsEventThreatDetectionSettingsService(self)
    self.organizations_rapidVulnerabilityDetectionSettings = self.OrganizationsRapidVulnerabilityDetectionSettingsService(self)
    self.organizations_securityHealthAnalyticsSettings = self.OrganizationsSecurityHealthAnalyticsSettingsService(self)
    self.organizations_virtualMachineThreatDetectionSettings = self.OrganizationsVirtualMachineThreatDetectionSettingsService(self)
    self.organizations_webSecurityScannerSettings = self.OrganizationsWebSecurityScannerSettingsService(self)
    self.organizations = self.OrganizationsService(self)
    self.projects_containerThreatDetectionSettings = self.ProjectsContainerThreatDetectionSettingsService(self)
    self.projects_eventThreatDetectionSettings = self.ProjectsEventThreatDetectionSettingsService(self)
    self.projects_locations_clusters_containerThreatDetectionSettings = self.ProjectsLocationsClustersContainerThreatDetectionSettingsService(self)
    self.projects_locations_clusters = self.ProjectsLocationsClustersService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects_rapidVulnerabilityDetectionSettings = self.ProjectsRapidVulnerabilityDetectionSettingsService(self)
    self.projects_securityHealthAnalyticsSettings = self.ProjectsSecurityHealthAnalyticsSettingsService(self)
    self.projects_virtualMachineThreatDetectionSettings = self.ProjectsVirtualMachineThreatDetectionSettingsService(self)
    self.projects_webSecurityScannerSettings = self.ProjectsWebSecurityScannerSettingsService(self)
    self.projects = self.ProjectsService(self)

  class FoldersContainerThreatDetectionSettingsService(base_api.BaseApiService):
    """Service class for the folders_containerThreatDetectionSettings resource."""

    _NAME = 'folders_containerThreatDetectionSettings'

    def __init__(self, client):
      super(SecuritycenterV1beta2.FoldersContainerThreatDetectionSettingsService, self).__init__(client)
      self._upload_configs = {
          }

    def Calculate(self, request, global_params=None):
      r"""Calculates the effective ContainerThreatDetectionSettings based on its level in the resource hierarchy and its settings.

      Args:
        request: (SecuritycenterFoldersContainerThreatDetectionSettingsCalculateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ContainerThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('Calculate')
      return self._RunMethod(
          config, request, global_params=global_params)

    Calculate.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/folders/{foldersId}/containerThreatDetectionSettings:calculate',
        http_method='GET',
        method_id='securitycenter.folders.containerThreatDetectionSettings.calculate',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:calculate',
        request_field='',
        request_type_name='SecuritycenterFoldersContainerThreatDetectionSettingsCalculateRequest',
        response_type_name='ContainerThreatDetectionSettings',
        supports_download=False,
    )

  class FoldersEventThreatDetectionSettingsService(base_api.BaseApiService):
    """Service class for the folders_eventThreatDetectionSettings resource."""

    _NAME = 'folders_eventThreatDetectionSettings'

    def __init__(self, client):
      super(SecuritycenterV1beta2.FoldersEventThreatDetectionSettingsService, self).__init__(client)
      self._upload_configs = {
          }

    def Calculate(self, request, global_params=None):
      r"""Calculates the effective EventThreatDetectionSettings based on its level in the resource hierarchy and its settings.

      Args:
        request: (SecuritycenterFoldersEventThreatDetectionSettingsCalculateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (EventThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('Calculate')
      return self._RunMethod(
          config, request, global_params=global_params)

    Calculate.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/folders/{foldersId}/eventThreatDetectionSettings:calculate',
        http_method='GET',
        method_id='securitycenter.folders.eventThreatDetectionSettings.calculate',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:calculate',
        request_field='',
        request_type_name='SecuritycenterFoldersEventThreatDetectionSettingsCalculateRequest',
        response_type_name='EventThreatDetectionSettings',
        supports_download=False,
    )

  class FoldersRapidVulnerabilityDetectionSettingsService(base_api.BaseApiService):
    """Service class for the folders_rapidVulnerabilityDetectionSettings resource."""

    _NAME = 'folders_rapidVulnerabilityDetectionSettings'

    def __init__(self, client):
      super(SecuritycenterV1beta2.FoldersRapidVulnerabilityDetectionSettingsService, self).__init__(client)
      self._upload_configs = {
          }

    def Calculate(self, request, global_params=None):
      r"""Calculates the effective RapidVulnerabilityDetectionSettings based on its level in the resource hierarchy and its settings.

      Args:
        request: (SecuritycenterFoldersRapidVulnerabilityDetectionSettingsCalculateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (RapidVulnerabilityDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('Calculate')
      return self._RunMethod(
          config, request, global_params=global_params)

    Calculate.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/folders/{foldersId}/rapidVulnerabilityDetectionSettings:calculate',
        http_method='GET',
        method_id='securitycenter.folders.rapidVulnerabilityDetectionSettings.calculate',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:calculate',
        request_field='',
        request_type_name='SecuritycenterFoldersRapidVulnerabilityDetectionSettingsCalculateRequest',
        response_type_name='RapidVulnerabilityDetectionSettings',
        supports_download=False,
    )

  class FoldersSecurityHealthAnalyticsSettingsService(base_api.BaseApiService):
    """Service class for the folders_securityHealthAnalyticsSettings resource."""

    _NAME = 'folders_securityHealthAnalyticsSettings'

    def __init__(self, client):
      super(SecuritycenterV1beta2.FoldersSecurityHealthAnalyticsSettingsService, self).__init__(client)
      self._upload_configs = {
          }

    def Calculate(self, request, global_params=None):
      r"""Calculates the effective SecurityHealthAnalyticsSettings based on its level in the resource hierarchy and its settings.

      Args:
        request: (SecuritycenterFoldersSecurityHealthAnalyticsSettingsCalculateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SecurityHealthAnalyticsSettings) The response message.
      """
      config = self.GetMethodConfig('Calculate')
      return self._RunMethod(
          config, request, global_params=global_params)

    Calculate.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/folders/{foldersId}/securityHealthAnalyticsSettings:calculate',
        http_method='GET',
        method_id='securitycenter.folders.securityHealthAnalyticsSettings.calculate',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:calculate',
        request_field='',
        request_type_name='SecuritycenterFoldersSecurityHealthAnalyticsSettingsCalculateRequest',
        response_type_name='SecurityHealthAnalyticsSettings',
        supports_download=False,
    )

  class FoldersVirtualMachineThreatDetectionSettingsService(base_api.BaseApiService):
    """Service class for the folders_virtualMachineThreatDetectionSettings resource."""

    _NAME = 'folders_virtualMachineThreatDetectionSettings'

    def __init__(self, client):
      super(SecuritycenterV1beta2.FoldersVirtualMachineThreatDetectionSettingsService, self).__init__(client)
      self._upload_configs = {
          }

    def Calculate(self, request, global_params=None):
      r"""Calculates the effective VirtualMachineThreatDetectionSettings based on its level in the resource hierarchy and its settings.

      Args:
        request: (SecuritycenterFoldersVirtualMachineThreatDetectionSettingsCalculateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (VirtualMachineThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('Calculate')
      return self._RunMethod(
          config, request, global_params=global_params)

    Calculate.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/folders/{foldersId}/virtualMachineThreatDetectionSettings:calculate',
        http_method='GET',
        method_id='securitycenter.folders.virtualMachineThreatDetectionSettings.calculate',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:calculate',
        request_field='',
        request_type_name='SecuritycenterFoldersVirtualMachineThreatDetectionSettingsCalculateRequest',
        response_type_name='VirtualMachineThreatDetectionSettings',
        supports_download=False,
    )

  class FoldersWebSecurityScannerSettingsService(base_api.BaseApiService):
    """Service class for the folders_webSecurityScannerSettings resource."""

    _NAME = 'folders_webSecurityScannerSettings'

    def __init__(self, client):
      super(SecuritycenterV1beta2.FoldersWebSecurityScannerSettingsService, self).__init__(client)
      self._upload_configs = {
          }

    def Calculate(self, request, global_params=None):
      r"""Calculates the effective WebSecurityScannerSettings based on its level in the resource hierarchy and its settings.

      Args:
        request: (SecuritycenterFoldersWebSecurityScannerSettingsCalculateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (WebSecurityScannerSettings) The response message.
      """
      config = self.GetMethodConfig('Calculate')
      return self._RunMethod(
          config, request, global_params=global_params)

    Calculate.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/folders/{foldersId}/webSecurityScannerSettings:calculate',
        http_method='GET',
        method_id='securitycenter.folders.webSecurityScannerSettings.calculate',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:calculate',
        request_field='',
        request_type_name='SecuritycenterFoldersWebSecurityScannerSettingsCalculateRequest',
        response_type_name='WebSecurityScannerSettings',
        supports_download=False,
    )

  class FoldersService(base_api.BaseApiService):
    """Service class for the folders resource."""

    _NAME = 'folders'

    def __init__(self, client):
      super(SecuritycenterV1beta2.FoldersService, self).__init__(client)
      self._upload_configs = {
          }

    def GetContainerThreatDetectionSettings(self, request, global_params=None):
      r"""Get the ContainerThreatDetectionSettings resource.

      Args:
        request: (SecuritycenterFoldersGetContainerThreatDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ContainerThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('GetContainerThreatDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetContainerThreatDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/folders/{foldersId}/containerThreatDetectionSettings',
        http_method='GET',
        method_id='securitycenter.folders.getContainerThreatDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterFoldersGetContainerThreatDetectionSettingsRequest',
        response_type_name='ContainerThreatDetectionSettings',
        supports_download=False,
    )

    def GetEventThreatDetectionSettings(self, request, global_params=None):
      r"""Get the EventThreatDetectionSettings resource.

      Args:
        request: (SecuritycenterFoldersGetEventThreatDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (EventThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('GetEventThreatDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetEventThreatDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/folders/{foldersId}/eventThreatDetectionSettings',
        http_method='GET',
        method_id='securitycenter.folders.getEventThreatDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterFoldersGetEventThreatDetectionSettingsRequest',
        response_type_name='EventThreatDetectionSettings',
        supports_download=False,
    )

    def GetOnboardingState(self, request, global_params=None):
      r"""Retrieve the OnboardingState of a resource.

      Args:
        request: (SecuritycenterFoldersGetOnboardingStateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (OnboardingState) The response message.
      """
      config = self.GetMethodConfig('GetOnboardingState')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetOnboardingState.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/folders/{foldersId}/onboardingState',
        http_method='GET',
        method_id='securitycenter.folders.getOnboardingState',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterFoldersGetOnboardingStateRequest',
        response_type_name='OnboardingState',
        supports_download=False,
    )

    def GetRapidVulnerabilityDetectionSettings(self, request, global_params=None):
      r"""Get the RapidVulnerabilityDetectionSettings resource.

      Args:
        request: (SecuritycenterFoldersGetRapidVulnerabilityDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (RapidVulnerabilityDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('GetRapidVulnerabilityDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetRapidVulnerabilityDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/folders/{foldersId}/rapidVulnerabilityDetectionSettings',
        http_method='GET',
        method_id='securitycenter.folders.getRapidVulnerabilityDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterFoldersGetRapidVulnerabilityDetectionSettingsRequest',
        response_type_name='RapidVulnerabilityDetectionSettings',
        supports_download=False,
    )

    def GetSecurityCenterSettings(self, request, global_params=None):
      r"""Get the SecurityCenterSettings resource.

      Args:
        request: (SecuritycenterFoldersGetSecurityCenterSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SecurityCenterSettings) The response message.
      """
      config = self.GetMethodConfig('GetSecurityCenterSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetSecurityCenterSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/folders/{foldersId}/securityCenterSettings',
        http_method='GET',
        method_id='securitycenter.folders.getSecurityCenterSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterFoldersGetSecurityCenterSettingsRequest',
        response_type_name='SecurityCenterSettings',
        supports_download=False,
    )

    def GetSecurityHealthAnalyticsSettings(self, request, global_params=None):
      r"""Get the SecurityHealthAnalyticsSettings resource.

      Args:
        request: (SecuritycenterFoldersGetSecurityHealthAnalyticsSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SecurityHealthAnalyticsSettings) The response message.
      """
      config = self.GetMethodConfig('GetSecurityHealthAnalyticsSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetSecurityHealthAnalyticsSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/folders/{foldersId}/securityHealthAnalyticsSettings',
        http_method='GET',
        method_id='securitycenter.folders.getSecurityHealthAnalyticsSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterFoldersGetSecurityHealthAnalyticsSettingsRequest',
        response_type_name='SecurityHealthAnalyticsSettings',
        supports_download=False,
    )

    def GetVirtualMachineThreatDetectionSettings(self, request, global_params=None):
      r"""Get the VirtualMachineThreatDetectionSettings resource.

      Args:
        request: (SecuritycenterFoldersGetVirtualMachineThreatDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (VirtualMachineThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('GetVirtualMachineThreatDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetVirtualMachineThreatDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/folders/{foldersId}/virtualMachineThreatDetectionSettings',
        http_method='GET',
        method_id='securitycenter.folders.getVirtualMachineThreatDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterFoldersGetVirtualMachineThreatDetectionSettingsRequest',
        response_type_name='VirtualMachineThreatDetectionSettings',
        supports_download=False,
    )

    def GetWebSecurityScannerSettings(self, request, global_params=None):
      r"""Get the WebSecurityScannerSettings resource.

      Args:
        request: (SecuritycenterFoldersGetWebSecurityScannerSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (WebSecurityScannerSettings) The response message.
      """
      config = self.GetMethodConfig('GetWebSecurityScannerSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetWebSecurityScannerSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/folders/{foldersId}/webSecurityScannerSettings',
        http_method='GET',
        method_id='securitycenter.folders.getWebSecurityScannerSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterFoldersGetWebSecurityScannerSettingsRequest',
        response_type_name='WebSecurityScannerSettings',
        supports_download=False,
    )

    def UpdateContainerThreatDetectionSettings(self, request, global_params=None):
      r"""Update the ContainerThreatDetectionSettings resource.

      Args:
        request: (SecuritycenterFoldersUpdateContainerThreatDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ContainerThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('UpdateContainerThreatDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateContainerThreatDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/folders/{foldersId}/containerThreatDetectionSettings',
        http_method='PATCH',
        method_id='securitycenter.folders.updateContainerThreatDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta2/{+name}',
        request_field='containerThreatDetectionSettings',
        request_type_name='SecuritycenterFoldersUpdateContainerThreatDetectionSettingsRequest',
        response_type_name='ContainerThreatDetectionSettings',
        supports_download=False,
    )

    def UpdateEventThreatDetectionSettings(self, request, global_params=None):
      r"""Update the EventThreatDetectionSettings resource.

      Args:
        request: (SecuritycenterFoldersUpdateEventThreatDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (EventThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('UpdateEventThreatDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateEventThreatDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/folders/{foldersId}/eventThreatDetectionSettings',
        http_method='PATCH',
        method_id='securitycenter.folders.updateEventThreatDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta2/{+name}',
        request_field='eventThreatDetectionSettings',
        request_type_name='SecuritycenterFoldersUpdateEventThreatDetectionSettingsRequest',
        response_type_name='EventThreatDetectionSettings',
        supports_download=False,
    )

    def UpdateRapidVulnerabilityDetectionSettings(self, request, global_params=None):
      r"""Update the RapidVulnerabilityDetectionSettings resource.

      Args:
        request: (SecuritycenterFoldersUpdateRapidVulnerabilityDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (RapidVulnerabilityDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('UpdateRapidVulnerabilityDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateRapidVulnerabilityDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/folders/{foldersId}/rapidVulnerabilityDetectionSettings',
        http_method='PATCH',
        method_id='securitycenter.folders.updateRapidVulnerabilityDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta2/{+name}',
        request_field='rapidVulnerabilityDetectionSettings',
        request_type_name='SecuritycenterFoldersUpdateRapidVulnerabilityDetectionSettingsRequest',
        response_type_name='RapidVulnerabilityDetectionSettings',
        supports_download=False,
    )

    def UpdateSecurityHealthAnalyticsSettings(self, request, global_params=None):
      r"""Update the SecurityHealthAnalyticsSettings resource.

      Args:
        request: (SecuritycenterFoldersUpdateSecurityHealthAnalyticsSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SecurityHealthAnalyticsSettings) The response message.
      """
      config = self.GetMethodConfig('UpdateSecurityHealthAnalyticsSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateSecurityHealthAnalyticsSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/folders/{foldersId}/securityHealthAnalyticsSettings',
        http_method='PATCH',
        method_id='securitycenter.folders.updateSecurityHealthAnalyticsSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta2/{+name}',
        request_field='securityHealthAnalyticsSettings',
        request_type_name='SecuritycenterFoldersUpdateSecurityHealthAnalyticsSettingsRequest',
        response_type_name='SecurityHealthAnalyticsSettings',
        supports_download=False,
    )

    def UpdateVirtualMachineThreatDetectionSettings(self, request, global_params=None):
      r"""Update the VirtualMachineThreatDetectionSettings resource.

      Args:
        request: (SecuritycenterFoldersUpdateVirtualMachineThreatDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (VirtualMachineThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('UpdateVirtualMachineThreatDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateVirtualMachineThreatDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/folders/{foldersId}/virtualMachineThreatDetectionSettings',
        http_method='PATCH',
        method_id='securitycenter.folders.updateVirtualMachineThreatDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta2/{+name}',
        request_field='virtualMachineThreatDetectionSettings',
        request_type_name='SecuritycenterFoldersUpdateVirtualMachineThreatDetectionSettingsRequest',
        response_type_name='VirtualMachineThreatDetectionSettings',
        supports_download=False,
    )

    def UpdateWebSecurityScannerSettings(self, request, global_params=None):
      r"""Update the WebSecurityScannerSettings resource.

      Args:
        request: (SecuritycenterFoldersUpdateWebSecurityScannerSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (WebSecurityScannerSettings) The response message.
      """
      config = self.GetMethodConfig('UpdateWebSecurityScannerSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateWebSecurityScannerSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/folders/{foldersId}/webSecurityScannerSettings',
        http_method='PATCH',
        method_id='securitycenter.folders.updateWebSecurityScannerSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta2/{+name}',
        request_field='webSecurityScannerSettings',
        request_type_name='SecuritycenterFoldersUpdateWebSecurityScannerSettingsRequest',
        response_type_name='WebSecurityScannerSettings',
        supports_download=False,
    )

  class OrganizationsContainerThreatDetectionSettingsService(base_api.BaseApiService):
    """Service class for the organizations_containerThreatDetectionSettings resource."""

    _NAME = 'organizations_containerThreatDetectionSettings'

    def __init__(self, client):
      super(SecuritycenterV1beta2.OrganizationsContainerThreatDetectionSettingsService, self).__init__(client)
      self._upload_configs = {
          }

    def Calculate(self, request, global_params=None):
      r"""Calculates the effective ContainerThreatDetectionSettings based on its level in the resource hierarchy and its settings.

      Args:
        request: (SecuritycenterOrganizationsContainerThreatDetectionSettingsCalculateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ContainerThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('Calculate')
      return self._RunMethod(
          config, request, global_params=global_params)

    Calculate.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/organizations/{organizationsId}/containerThreatDetectionSettings:calculate',
        http_method='GET',
        method_id='securitycenter.organizations.containerThreatDetectionSettings.calculate',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:calculate',
        request_field='',
        request_type_name='SecuritycenterOrganizationsContainerThreatDetectionSettingsCalculateRequest',
        response_type_name='ContainerThreatDetectionSettings',
        supports_download=False,
    )

  class OrganizationsEventThreatDetectionSettingsService(base_api.BaseApiService):
    """Service class for the organizations_eventThreatDetectionSettings resource."""

    _NAME = 'organizations_eventThreatDetectionSettings'

    def __init__(self, client):
      super(SecuritycenterV1beta2.OrganizationsEventThreatDetectionSettingsService, self).__init__(client)
      self._upload_configs = {
          }

    def Calculate(self, request, global_params=None):
      r"""Calculates the effective EventThreatDetectionSettings based on its level in the resource hierarchy and its settings.

      Args:
        request: (SecuritycenterOrganizationsEventThreatDetectionSettingsCalculateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (EventThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('Calculate')
      return self._RunMethod(
          config, request, global_params=global_params)

    Calculate.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/organizations/{organizationsId}/eventThreatDetectionSettings:calculate',
        http_method='GET',
        method_id='securitycenter.organizations.eventThreatDetectionSettings.calculate',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:calculate',
        request_field='',
        request_type_name='SecuritycenterOrganizationsEventThreatDetectionSettingsCalculateRequest',
        response_type_name='EventThreatDetectionSettings',
        supports_download=False,
    )

  class OrganizationsRapidVulnerabilityDetectionSettingsService(base_api.BaseApiService):
    """Service class for the organizations_rapidVulnerabilityDetectionSettings resource."""

    _NAME = 'organizations_rapidVulnerabilityDetectionSettings'

    def __init__(self, client):
      super(SecuritycenterV1beta2.OrganizationsRapidVulnerabilityDetectionSettingsService, self).__init__(client)
      self._upload_configs = {
          }

    def Calculate(self, request, global_params=None):
      r"""Calculates the effective RapidVulnerabilityDetectionSettings based on its level in the resource hierarchy and its settings.

      Args:
        request: (SecuritycenterOrganizationsRapidVulnerabilityDetectionSettingsCalculateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (RapidVulnerabilityDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('Calculate')
      return self._RunMethod(
          config, request, global_params=global_params)

    Calculate.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/organizations/{organizationsId}/rapidVulnerabilityDetectionSettings:calculate',
        http_method='GET',
        method_id='securitycenter.organizations.rapidVulnerabilityDetectionSettings.calculate',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:calculate',
        request_field='',
        request_type_name='SecuritycenterOrganizationsRapidVulnerabilityDetectionSettingsCalculateRequest',
        response_type_name='RapidVulnerabilityDetectionSettings',
        supports_download=False,
    )

  class OrganizationsSecurityHealthAnalyticsSettingsService(base_api.BaseApiService):
    """Service class for the organizations_securityHealthAnalyticsSettings resource."""

    _NAME = 'organizations_securityHealthAnalyticsSettings'

    def __init__(self, client):
      super(SecuritycenterV1beta2.OrganizationsSecurityHealthAnalyticsSettingsService, self).__init__(client)
      self._upload_configs = {
          }

    def Calculate(self, request, global_params=None):
      r"""Calculates the effective SecurityHealthAnalyticsSettings based on its level in the resource hierarchy and its settings.

      Args:
        request: (SecuritycenterOrganizationsSecurityHealthAnalyticsSettingsCalculateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SecurityHealthAnalyticsSettings) The response message.
      """
      config = self.GetMethodConfig('Calculate')
      return self._RunMethod(
          config, request, global_params=global_params)

    Calculate.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/organizations/{organizationsId}/securityHealthAnalyticsSettings:calculate',
        http_method='GET',
        method_id='securitycenter.organizations.securityHealthAnalyticsSettings.calculate',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:calculate',
        request_field='',
        request_type_name='SecuritycenterOrganizationsSecurityHealthAnalyticsSettingsCalculateRequest',
        response_type_name='SecurityHealthAnalyticsSettings',
        supports_download=False,
    )

  class OrganizationsVirtualMachineThreatDetectionSettingsService(base_api.BaseApiService):
    """Service class for the organizations_virtualMachineThreatDetectionSettings resource."""

    _NAME = 'organizations_virtualMachineThreatDetectionSettings'

    def __init__(self, client):
      super(SecuritycenterV1beta2.OrganizationsVirtualMachineThreatDetectionSettingsService, self).__init__(client)
      self._upload_configs = {
          }

    def Calculate(self, request, global_params=None):
      r"""Calculates the effective VirtualMachineThreatDetectionSettings based on its level in the resource hierarchy and its settings.

      Args:
        request: (SecuritycenterOrganizationsVirtualMachineThreatDetectionSettingsCalculateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (VirtualMachineThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('Calculate')
      return self._RunMethod(
          config, request, global_params=global_params)

    Calculate.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/organizations/{organizationsId}/virtualMachineThreatDetectionSettings:calculate',
        http_method='GET',
        method_id='securitycenter.organizations.virtualMachineThreatDetectionSettings.calculate',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:calculate',
        request_field='',
        request_type_name='SecuritycenterOrganizationsVirtualMachineThreatDetectionSettingsCalculateRequest',
        response_type_name='VirtualMachineThreatDetectionSettings',
        supports_download=False,
    )

  class OrganizationsWebSecurityScannerSettingsService(base_api.BaseApiService):
    """Service class for the organizations_webSecurityScannerSettings resource."""

    _NAME = 'organizations_webSecurityScannerSettings'

    def __init__(self, client):
      super(SecuritycenterV1beta2.OrganizationsWebSecurityScannerSettingsService, self).__init__(client)
      self._upload_configs = {
          }

    def Calculate(self, request, global_params=None):
      r"""Calculates the effective WebSecurityScannerSettings based on its level in the resource hierarchy and its settings.

      Args:
        request: (SecuritycenterOrganizationsWebSecurityScannerSettingsCalculateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (WebSecurityScannerSettings) The response message.
      """
      config = self.GetMethodConfig('Calculate')
      return self._RunMethod(
          config, request, global_params=global_params)

    Calculate.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/organizations/{organizationsId}/webSecurityScannerSettings:calculate',
        http_method='GET',
        method_id='securitycenter.organizations.webSecurityScannerSettings.calculate',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:calculate',
        request_field='',
        request_type_name='SecuritycenterOrganizationsWebSecurityScannerSettingsCalculateRequest',
        response_type_name='WebSecurityScannerSettings',
        supports_download=False,
    )

  class OrganizationsService(base_api.BaseApiService):
    """Service class for the organizations resource."""

    _NAME = 'organizations'

    def __init__(self, client):
      super(SecuritycenterV1beta2.OrganizationsService, self).__init__(client)
      self._upload_configs = {
          }

    def GetContainerThreatDetectionSettings(self, request, global_params=None):
      r"""Get the ContainerThreatDetectionSettings resource.

      Args:
        request: (SecuritycenterOrganizationsGetContainerThreatDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ContainerThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('GetContainerThreatDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetContainerThreatDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/organizations/{organizationsId}/containerThreatDetectionSettings',
        http_method='GET',
        method_id='securitycenter.organizations.getContainerThreatDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterOrganizationsGetContainerThreatDetectionSettingsRequest',
        response_type_name='ContainerThreatDetectionSettings',
        supports_download=False,
    )

    def GetEventThreatDetectionSettings(self, request, global_params=None):
      r"""Get the EventThreatDetectionSettings resource.

      Args:
        request: (SecuritycenterOrganizationsGetEventThreatDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (EventThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('GetEventThreatDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetEventThreatDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/organizations/{organizationsId}/eventThreatDetectionSettings',
        http_method='GET',
        method_id='securitycenter.organizations.getEventThreatDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterOrganizationsGetEventThreatDetectionSettingsRequest',
        response_type_name='EventThreatDetectionSettings',
        supports_download=False,
    )

    def GetOnboardingState(self, request, global_params=None):
      r"""Retrieve the OnboardingState of a resource.

      Args:
        request: (SecuritycenterOrganizationsGetOnboardingStateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (OnboardingState) The response message.
      """
      config = self.GetMethodConfig('GetOnboardingState')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetOnboardingState.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/organizations/{organizationsId}/onboardingState',
        http_method='GET',
        method_id='securitycenter.organizations.getOnboardingState',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterOrganizationsGetOnboardingStateRequest',
        response_type_name='OnboardingState',
        supports_download=False,
    )

    def GetRapidVulnerabilityDetectionSettings(self, request, global_params=None):
      r"""Get the RapidVulnerabilityDetectionSettings resource.

      Args:
        request: (SecuritycenterOrganizationsGetRapidVulnerabilityDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (RapidVulnerabilityDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('GetRapidVulnerabilityDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetRapidVulnerabilityDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/organizations/{organizationsId}/rapidVulnerabilityDetectionSettings',
        http_method='GET',
        method_id='securitycenter.organizations.getRapidVulnerabilityDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterOrganizationsGetRapidVulnerabilityDetectionSettingsRequest',
        response_type_name='RapidVulnerabilityDetectionSettings',
        supports_download=False,
    )

    def GetSecurityCenterSettings(self, request, global_params=None):
      r"""Get the SecurityCenterSettings resource.

      Args:
        request: (SecuritycenterOrganizationsGetSecurityCenterSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SecurityCenterSettings) The response message.
      """
      config = self.GetMethodConfig('GetSecurityCenterSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetSecurityCenterSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/organizations/{organizationsId}/securityCenterSettings',
        http_method='GET',
        method_id='securitycenter.organizations.getSecurityCenterSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterOrganizationsGetSecurityCenterSettingsRequest',
        response_type_name='SecurityCenterSettings',
        supports_download=False,
    )

    def GetSecurityHealthAnalyticsSettings(self, request, global_params=None):
      r"""Get the SecurityHealthAnalyticsSettings resource.

      Args:
        request: (SecuritycenterOrganizationsGetSecurityHealthAnalyticsSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SecurityHealthAnalyticsSettings) The response message.
      """
      config = self.GetMethodConfig('GetSecurityHealthAnalyticsSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetSecurityHealthAnalyticsSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/organizations/{organizationsId}/securityHealthAnalyticsSettings',
        http_method='GET',
        method_id='securitycenter.organizations.getSecurityHealthAnalyticsSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterOrganizationsGetSecurityHealthAnalyticsSettingsRequest',
        response_type_name='SecurityHealthAnalyticsSettings',
        supports_download=False,
    )

    def GetSubscription(self, request, global_params=None):
      r"""Get the Subscription resource.

      Args:
        request: (SecuritycenterOrganizationsGetSubscriptionRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Subscription) The response message.
      """
      config = self.GetMethodConfig('GetSubscription')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetSubscription.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/organizations/{organizationsId}/subscription',
        http_method='GET',
        method_id='securitycenter.organizations.getSubscription',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterOrganizationsGetSubscriptionRequest',
        response_type_name='Subscription',
        supports_download=False,
    )

    def GetVirtualMachineThreatDetectionSettings(self, request, global_params=None):
      r"""Get the VirtualMachineThreatDetectionSettings resource.

      Args:
        request: (SecuritycenterOrganizationsGetVirtualMachineThreatDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (VirtualMachineThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('GetVirtualMachineThreatDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetVirtualMachineThreatDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/organizations/{organizationsId}/virtualMachineThreatDetectionSettings',
        http_method='GET',
        method_id='securitycenter.organizations.getVirtualMachineThreatDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterOrganizationsGetVirtualMachineThreatDetectionSettingsRequest',
        response_type_name='VirtualMachineThreatDetectionSettings',
        supports_download=False,
    )

    def GetWebSecurityScannerSettings(self, request, global_params=None):
      r"""Get the WebSecurityScannerSettings resource.

      Args:
        request: (SecuritycenterOrganizationsGetWebSecurityScannerSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (WebSecurityScannerSettings) The response message.
      """
      config = self.GetMethodConfig('GetWebSecurityScannerSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetWebSecurityScannerSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/organizations/{organizationsId}/webSecurityScannerSettings',
        http_method='GET',
        method_id='securitycenter.organizations.getWebSecurityScannerSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterOrganizationsGetWebSecurityScannerSettingsRequest',
        response_type_name='WebSecurityScannerSettings',
        supports_download=False,
    )

    def UpdateContainerThreatDetectionSettings(self, request, global_params=None):
      r"""Update the ContainerThreatDetectionSettings resource.

      Args:
        request: (SecuritycenterOrganizationsUpdateContainerThreatDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ContainerThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('UpdateContainerThreatDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateContainerThreatDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/organizations/{organizationsId}/containerThreatDetectionSettings',
        http_method='PATCH',
        method_id='securitycenter.organizations.updateContainerThreatDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta2/{+name}',
        request_field='containerThreatDetectionSettings',
        request_type_name='SecuritycenterOrganizationsUpdateContainerThreatDetectionSettingsRequest',
        response_type_name='ContainerThreatDetectionSettings',
        supports_download=False,
    )

    def UpdateEventThreatDetectionSettings(self, request, global_params=None):
      r"""Update the EventThreatDetectionSettings resource.

      Args:
        request: (SecuritycenterOrganizationsUpdateEventThreatDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (EventThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('UpdateEventThreatDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateEventThreatDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/organizations/{organizationsId}/eventThreatDetectionSettings',
        http_method='PATCH',
        method_id='securitycenter.organizations.updateEventThreatDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta2/{+name}',
        request_field='eventThreatDetectionSettings',
        request_type_name='SecuritycenterOrganizationsUpdateEventThreatDetectionSettingsRequest',
        response_type_name='EventThreatDetectionSettings',
        supports_download=False,
    )

    def UpdateRapidVulnerabilityDetectionSettings(self, request, global_params=None):
      r"""Update the RapidVulnerabilityDetectionSettings resource.

      Args:
        request: (SecuritycenterOrganizationsUpdateRapidVulnerabilityDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (RapidVulnerabilityDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('UpdateRapidVulnerabilityDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateRapidVulnerabilityDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/organizations/{organizationsId}/rapidVulnerabilityDetectionSettings',
        http_method='PATCH',
        method_id='securitycenter.organizations.updateRapidVulnerabilityDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta2/{+name}',
        request_field='rapidVulnerabilityDetectionSettings',
        request_type_name='SecuritycenterOrganizationsUpdateRapidVulnerabilityDetectionSettingsRequest',
        response_type_name='RapidVulnerabilityDetectionSettings',
        supports_download=False,
    )

    def UpdateSecurityHealthAnalyticsSettings(self, request, global_params=None):
      r"""Update the SecurityHealthAnalyticsSettings resource.

      Args:
        request: (SecuritycenterOrganizationsUpdateSecurityHealthAnalyticsSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SecurityHealthAnalyticsSettings) The response message.
      """
      config = self.GetMethodConfig('UpdateSecurityHealthAnalyticsSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateSecurityHealthAnalyticsSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/organizations/{organizationsId}/securityHealthAnalyticsSettings',
        http_method='PATCH',
        method_id='securitycenter.organizations.updateSecurityHealthAnalyticsSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta2/{+name}',
        request_field='securityHealthAnalyticsSettings',
        request_type_name='SecuritycenterOrganizationsUpdateSecurityHealthAnalyticsSettingsRequest',
        response_type_name='SecurityHealthAnalyticsSettings',
        supports_download=False,
    )

    def UpdateVirtualMachineThreatDetectionSettings(self, request, global_params=None):
      r"""Update the VirtualMachineThreatDetectionSettings resource.

      Args:
        request: (SecuritycenterOrganizationsUpdateVirtualMachineThreatDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (VirtualMachineThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('UpdateVirtualMachineThreatDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateVirtualMachineThreatDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/organizations/{organizationsId}/virtualMachineThreatDetectionSettings',
        http_method='PATCH',
        method_id='securitycenter.organizations.updateVirtualMachineThreatDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta2/{+name}',
        request_field='virtualMachineThreatDetectionSettings',
        request_type_name='SecuritycenterOrganizationsUpdateVirtualMachineThreatDetectionSettingsRequest',
        response_type_name='VirtualMachineThreatDetectionSettings',
        supports_download=False,
    )

    def UpdateWebSecurityScannerSettings(self, request, global_params=None):
      r"""Update the WebSecurityScannerSettings resource.

      Args:
        request: (SecuritycenterOrganizationsUpdateWebSecurityScannerSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (WebSecurityScannerSettings) The response message.
      """
      config = self.GetMethodConfig('UpdateWebSecurityScannerSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateWebSecurityScannerSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/organizations/{organizationsId}/webSecurityScannerSettings',
        http_method='PATCH',
        method_id='securitycenter.organizations.updateWebSecurityScannerSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta2/{+name}',
        request_field='webSecurityScannerSettings',
        request_type_name='SecuritycenterOrganizationsUpdateWebSecurityScannerSettingsRequest',
        response_type_name='WebSecurityScannerSettings',
        supports_download=False,
    )

  class ProjectsContainerThreatDetectionSettingsService(base_api.BaseApiService):
    """Service class for the projects_containerThreatDetectionSettings resource."""

    _NAME = 'projects_containerThreatDetectionSettings'

    def __init__(self, client):
      super(SecuritycenterV1beta2.ProjectsContainerThreatDetectionSettingsService, self).__init__(client)
      self._upload_configs = {
          }

    def Calculate(self, request, global_params=None):
      r"""Calculates the effective ContainerThreatDetectionSettings based on its level in the resource hierarchy and its settings.

      Args:
        request: (SecuritycenterProjectsContainerThreatDetectionSettingsCalculateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ContainerThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('Calculate')
      return self._RunMethod(
          config, request, global_params=global_params)

    Calculate.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/containerThreatDetectionSettings:calculate',
        http_method='GET',
        method_id='securitycenter.projects.containerThreatDetectionSettings.calculate',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:calculate',
        request_field='',
        request_type_name='SecuritycenterProjectsContainerThreatDetectionSettingsCalculateRequest',
        response_type_name='ContainerThreatDetectionSettings',
        supports_download=False,
    )

  class ProjectsEventThreatDetectionSettingsService(base_api.BaseApiService):
    """Service class for the projects_eventThreatDetectionSettings resource."""

    _NAME = 'projects_eventThreatDetectionSettings'

    def __init__(self, client):
      super(SecuritycenterV1beta2.ProjectsEventThreatDetectionSettingsService, self).__init__(client)
      self._upload_configs = {
          }

    def Calculate(self, request, global_params=None):
      r"""Calculates the effective EventThreatDetectionSettings based on its level in the resource hierarchy and its settings.

      Args:
        request: (SecuritycenterProjectsEventThreatDetectionSettingsCalculateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (EventThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('Calculate')
      return self._RunMethod(
          config, request, global_params=global_params)

    Calculate.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/eventThreatDetectionSettings:calculate',
        http_method='GET',
        method_id='securitycenter.projects.eventThreatDetectionSettings.calculate',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:calculate',
        request_field='',
        request_type_name='SecuritycenterProjectsEventThreatDetectionSettingsCalculateRequest',
        response_type_name='EventThreatDetectionSettings',
        supports_download=False,
    )

  class ProjectsLocationsClustersContainerThreatDetectionSettingsService(base_api.BaseApiService):
    """Service class for the projects_locations_clusters_containerThreatDetectionSettings resource."""

    _NAME = 'projects_locations_clusters_containerThreatDetectionSettings'

    def __init__(self, client):
      super(SecuritycenterV1beta2.ProjectsLocationsClustersContainerThreatDetectionSettingsService, self).__init__(client)
      self._upload_configs = {
          }

    def Calculate(self, request, global_params=None):
      r"""Calculates the effective ContainerThreatDetectionSettings based on its level in the resource hierarchy and its settings.

      Args:
        request: (SecuritycenterProjectsLocationsClustersContainerThreatDetectionSettingsCalculateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ContainerThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('Calculate')
      return self._RunMethod(
          config, request, global_params=global_params)

    Calculate.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/locations/{locationsId}/clusters/{clustersId}/containerThreatDetectionSettings:calculate',
        http_method='GET',
        method_id='securitycenter.projects.locations.clusters.containerThreatDetectionSettings.calculate',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:calculate',
        request_field='',
        request_type_name='SecuritycenterProjectsLocationsClustersContainerThreatDetectionSettingsCalculateRequest',
        response_type_name='ContainerThreatDetectionSettings',
        supports_download=False,
    )

  class ProjectsLocationsClustersService(base_api.BaseApiService):
    """Service class for the projects_locations_clusters resource."""

    _NAME = 'projects_locations_clusters'

    def __init__(self, client):
      super(SecuritycenterV1beta2.ProjectsLocationsClustersService, self).__init__(client)
      self._upload_configs = {
          }

    def GetContainerThreatDetectionSettings(self, request, global_params=None):
      r"""Get the ContainerThreatDetectionSettings resource.

      Args:
        request: (SecuritycenterProjectsLocationsClustersGetContainerThreatDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ContainerThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('GetContainerThreatDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetContainerThreatDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/locations/{locationsId}/clusters/{clustersId}/containerThreatDetectionSettings',
        http_method='GET',
        method_id='securitycenter.projects.locations.clusters.getContainerThreatDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterProjectsLocationsClustersGetContainerThreatDetectionSettingsRequest',
        response_type_name='ContainerThreatDetectionSettings',
        supports_download=False,
    )

    def UpdateContainerThreatDetectionSettings(self, request, global_params=None):
      r"""Update the ContainerThreatDetectionSettings resource.

      Args:
        request: (SecuritycenterProjectsLocationsClustersUpdateContainerThreatDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ContainerThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('UpdateContainerThreatDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateContainerThreatDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/locations/{locationsId}/clusters/{clustersId}/containerThreatDetectionSettings',
        http_method='PATCH',
        method_id='securitycenter.projects.locations.clusters.updateContainerThreatDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta2/{+name}',
        request_field='containerThreatDetectionSettings',
        request_type_name='SecuritycenterProjectsLocationsClustersUpdateContainerThreatDetectionSettingsRequest',
        response_type_name='ContainerThreatDetectionSettings',
        supports_download=False,
    )

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(SecuritycenterV1beta2.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

  class ProjectsRapidVulnerabilityDetectionSettingsService(base_api.BaseApiService):
    """Service class for the projects_rapidVulnerabilityDetectionSettings resource."""

    _NAME = 'projects_rapidVulnerabilityDetectionSettings'

    def __init__(self, client):
      super(SecuritycenterV1beta2.ProjectsRapidVulnerabilityDetectionSettingsService, self).__init__(client)
      self._upload_configs = {
          }

    def Calculate(self, request, global_params=None):
      r"""Calculates the effective RapidVulnerabilityDetectionSettings based on its level in the resource hierarchy and its settings.

      Args:
        request: (SecuritycenterProjectsRapidVulnerabilityDetectionSettingsCalculateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (RapidVulnerabilityDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('Calculate')
      return self._RunMethod(
          config, request, global_params=global_params)

    Calculate.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/rapidVulnerabilityDetectionSettings:calculate',
        http_method='GET',
        method_id='securitycenter.projects.rapidVulnerabilityDetectionSettings.calculate',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:calculate',
        request_field='',
        request_type_name='SecuritycenterProjectsRapidVulnerabilityDetectionSettingsCalculateRequest',
        response_type_name='RapidVulnerabilityDetectionSettings',
        supports_download=False,
    )

  class ProjectsSecurityHealthAnalyticsSettingsService(base_api.BaseApiService):
    """Service class for the projects_securityHealthAnalyticsSettings resource."""

    _NAME = 'projects_securityHealthAnalyticsSettings'

    def __init__(self, client):
      super(SecuritycenterV1beta2.ProjectsSecurityHealthAnalyticsSettingsService, self).__init__(client)
      self._upload_configs = {
          }

    def Calculate(self, request, global_params=None):
      r"""Calculates the effective SecurityHealthAnalyticsSettings based on its level in the resource hierarchy and its settings.

      Args:
        request: (SecuritycenterProjectsSecurityHealthAnalyticsSettingsCalculateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SecurityHealthAnalyticsSettings) The response message.
      """
      config = self.GetMethodConfig('Calculate')
      return self._RunMethod(
          config, request, global_params=global_params)

    Calculate.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/securityHealthAnalyticsSettings:calculate',
        http_method='GET',
        method_id='securitycenter.projects.securityHealthAnalyticsSettings.calculate',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:calculate',
        request_field='',
        request_type_name='SecuritycenterProjectsSecurityHealthAnalyticsSettingsCalculateRequest',
        response_type_name='SecurityHealthAnalyticsSettings',
        supports_download=False,
    )

  class ProjectsVirtualMachineThreatDetectionSettingsService(base_api.BaseApiService):
    """Service class for the projects_virtualMachineThreatDetectionSettings resource."""

    _NAME = 'projects_virtualMachineThreatDetectionSettings'

    def __init__(self, client):
      super(SecuritycenterV1beta2.ProjectsVirtualMachineThreatDetectionSettingsService, self).__init__(client)
      self._upload_configs = {
          }

    def Calculate(self, request, global_params=None):
      r"""Calculates the effective VirtualMachineThreatDetectionSettings based on its level in the resource hierarchy and its settings.

      Args:
        request: (SecuritycenterProjectsVirtualMachineThreatDetectionSettingsCalculateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (VirtualMachineThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('Calculate')
      return self._RunMethod(
          config, request, global_params=global_params)

    Calculate.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/virtualMachineThreatDetectionSettings:calculate',
        http_method='GET',
        method_id='securitycenter.projects.virtualMachineThreatDetectionSettings.calculate',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:calculate',
        request_field='',
        request_type_name='SecuritycenterProjectsVirtualMachineThreatDetectionSettingsCalculateRequest',
        response_type_name='VirtualMachineThreatDetectionSettings',
        supports_download=False,
    )

  class ProjectsWebSecurityScannerSettingsService(base_api.BaseApiService):
    """Service class for the projects_webSecurityScannerSettings resource."""

    _NAME = 'projects_webSecurityScannerSettings'

    def __init__(self, client):
      super(SecuritycenterV1beta2.ProjectsWebSecurityScannerSettingsService, self).__init__(client)
      self._upload_configs = {
          }

    def Calculate(self, request, global_params=None):
      r"""Calculates the effective WebSecurityScannerSettings based on its level in the resource hierarchy and its settings.

      Args:
        request: (SecuritycenterProjectsWebSecurityScannerSettingsCalculateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (WebSecurityScannerSettings) The response message.
      """
      config = self.GetMethodConfig('Calculate')
      return self._RunMethod(
          config, request, global_params=global_params)

    Calculate.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/webSecurityScannerSettings:calculate',
        http_method='GET',
        method_id='securitycenter.projects.webSecurityScannerSettings.calculate',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}:calculate',
        request_field='',
        request_type_name='SecuritycenterProjectsWebSecurityScannerSettingsCalculateRequest',
        response_type_name='WebSecurityScannerSettings',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(SecuritycenterV1beta2.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }

    def GetContainerThreatDetectionSettings(self, request, global_params=None):
      r"""Get the ContainerThreatDetectionSettings resource.

      Args:
        request: (SecuritycenterProjectsGetContainerThreatDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ContainerThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('GetContainerThreatDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetContainerThreatDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/containerThreatDetectionSettings',
        http_method='GET',
        method_id='securitycenter.projects.getContainerThreatDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterProjectsGetContainerThreatDetectionSettingsRequest',
        response_type_name='ContainerThreatDetectionSettings',
        supports_download=False,
    )

    def GetEventThreatDetectionSettings(self, request, global_params=None):
      r"""Get the EventThreatDetectionSettings resource.

      Args:
        request: (SecuritycenterProjectsGetEventThreatDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (EventThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('GetEventThreatDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetEventThreatDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/eventThreatDetectionSettings',
        http_method='GET',
        method_id='securitycenter.projects.getEventThreatDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterProjectsGetEventThreatDetectionSettingsRequest',
        response_type_name='EventThreatDetectionSettings',
        supports_download=False,
    )

    def GetOnboardingState(self, request, global_params=None):
      r"""Retrieve the OnboardingState of a resource.

      Args:
        request: (SecuritycenterProjectsGetOnboardingStateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (OnboardingState) The response message.
      """
      config = self.GetMethodConfig('GetOnboardingState')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetOnboardingState.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/onboardingState',
        http_method='GET',
        method_id='securitycenter.projects.getOnboardingState',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterProjectsGetOnboardingStateRequest',
        response_type_name='OnboardingState',
        supports_download=False,
    )

    def GetRapidVulnerabilityDetectionSettings(self, request, global_params=None):
      r"""Get the RapidVulnerabilityDetectionSettings resource.

      Args:
        request: (SecuritycenterProjectsGetRapidVulnerabilityDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (RapidVulnerabilityDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('GetRapidVulnerabilityDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetRapidVulnerabilityDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/rapidVulnerabilityDetectionSettings',
        http_method='GET',
        method_id='securitycenter.projects.getRapidVulnerabilityDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterProjectsGetRapidVulnerabilityDetectionSettingsRequest',
        response_type_name='RapidVulnerabilityDetectionSettings',
        supports_download=False,
    )

    def GetSecurityCenterSettings(self, request, global_params=None):
      r"""Get the SecurityCenterSettings resource.

      Args:
        request: (SecuritycenterProjectsGetSecurityCenterSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SecurityCenterSettings) The response message.
      """
      config = self.GetMethodConfig('GetSecurityCenterSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetSecurityCenterSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/securityCenterSettings',
        http_method='GET',
        method_id='securitycenter.projects.getSecurityCenterSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterProjectsGetSecurityCenterSettingsRequest',
        response_type_name='SecurityCenterSettings',
        supports_download=False,
    )

    def GetSecurityHealthAnalyticsSettings(self, request, global_params=None):
      r"""Get the SecurityHealthAnalyticsSettings resource.

      Args:
        request: (SecuritycenterProjectsGetSecurityHealthAnalyticsSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SecurityHealthAnalyticsSettings) The response message.
      """
      config = self.GetMethodConfig('GetSecurityHealthAnalyticsSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetSecurityHealthAnalyticsSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/securityHealthAnalyticsSettings',
        http_method='GET',
        method_id='securitycenter.projects.getSecurityHealthAnalyticsSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterProjectsGetSecurityHealthAnalyticsSettingsRequest',
        response_type_name='SecurityHealthAnalyticsSettings',
        supports_download=False,
    )

    def GetVirtualMachineThreatDetectionSettings(self, request, global_params=None):
      r"""Get the VirtualMachineThreatDetectionSettings resource.

      Args:
        request: (SecuritycenterProjectsGetVirtualMachineThreatDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (VirtualMachineThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('GetVirtualMachineThreatDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetVirtualMachineThreatDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/virtualMachineThreatDetectionSettings',
        http_method='GET',
        method_id='securitycenter.projects.getVirtualMachineThreatDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterProjectsGetVirtualMachineThreatDetectionSettingsRequest',
        response_type_name='VirtualMachineThreatDetectionSettings',
        supports_download=False,
    )

    def GetWebSecurityScannerSettings(self, request, global_params=None):
      r"""Get the WebSecurityScannerSettings resource.

      Args:
        request: (SecuritycenterProjectsGetWebSecurityScannerSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (WebSecurityScannerSettings) The response message.
      """
      config = self.GetMethodConfig('GetWebSecurityScannerSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetWebSecurityScannerSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/webSecurityScannerSettings',
        http_method='GET',
        method_id='securitycenter.projects.getWebSecurityScannerSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta2/{+name}',
        request_field='',
        request_type_name='SecuritycenterProjectsGetWebSecurityScannerSettingsRequest',
        response_type_name='WebSecurityScannerSettings',
        supports_download=False,
    )

    def UpdateContainerThreatDetectionSettings(self, request, global_params=None):
      r"""Update the ContainerThreatDetectionSettings resource.

      Args:
        request: (SecuritycenterProjectsUpdateContainerThreatDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ContainerThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('UpdateContainerThreatDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateContainerThreatDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/containerThreatDetectionSettings',
        http_method='PATCH',
        method_id='securitycenter.projects.updateContainerThreatDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta2/{+name}',
        request_field='containerThreatDetectionSettings',
        request_type_name='SecuritycenterProjectsUpdateContainerThreatDetectionSettingsRequest',
        response_type_name='ContainerThreatDetectionSettings',
        supports_download=False,
    )

    def UpdateEventThreatDetectionSettings(self, request, global_params=None):
      r"""Update the EventThreatDetectionSettings resource.

      Args:
        request: (SecuritycenterProjectsUpdateEventThreatDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (EventThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('UpdateEventThreatDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateEventThreatDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/eventThreatDetectionSettings',
        http_method='PATCH',
        method_id='securitycenter.projects.updateEventThreatDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta2/{+name}',
        request_field='eventThreatDetectionSettings',
        request_type_name='SecuritycenterProjectsUpdateEventThreatDetectionSettingsRequest',
        response_type_name='EventThreatDetectionSettings',
        supports_download=False,
    )

    def UpdateRapidVulnerabilityDetectionSettings(self, request, global_params=None):
      r"""Update the RapidVulnerabilityDetectionSettings resource.

      Args:
        request: (SecuritycenterProjectsUpdateRapidVulnerabilityDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (RapidVulnerabilityDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('UpdateRapidVulnerabilityDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateRapidVulnerabilityDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/rapidVulnerabilityDetectionSettings',
        http_method='PATCH',
        method_id='securitycenter.projects.updateRapidVulnerabilityDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta2/{+name}',
        request_field='rapidVulnerabilityDetectionSettings',
        request_type_name='SecuritycenterProjectsUpdateRapidVulnerabilityDetectionSettingsRequest',
        response_type_name='RapidVulnerabilityDetectionSettings',
        supports_download=False,
    )

    def UpdateSecurityHealthAnalyticsSettings(self, request, global_params=None):
      r"""Update the SecurityHealthAnalyticsSettings resource.

      Args:
        request: (SecuritycenterProjectsUpdateSecurityHealthAnalyticsSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SecurityHealthAnalyticsSettings) The response message.
      """
      config = self.GetMethodConfig('UpdateSecurityHealthAnalyticsSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateSecurityHealthAnalyticsSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/securityHealthAnalyticsSettings',
        http_method='PATCH',
        method_id='securitycenter.projects.updateSecurityHealthAnalyticsSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta2/{+name}',
        request_field='securityHealthAnalyticsSettings',
        request_type_name='SecuritycenterProjectsUpdateSecurityHealthAnalyticsSettingsRequest',
        response_type_name='SecurityHealthAnalyticsSettings',
        supports_download=False,
    )

    def UpdateVirtualMachineThreatDetectionSettings(self, request, global_params=None):
      r"""Update the VirtualMachineThreatDetectionSettings resource.

      Args:
        request: (SecuritycenterProjectsUpdateVirtualMachineThreatDetectionSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (VirtualMachineThreatDetectionSettings) The response message.
      """
      config = self.GetMethodConfig('UpdateVirtualMachineThreatDetectionSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateVirtualMachineThreatDetectionSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/virtualMachineThreatDetectionSettings',
        http_method='PATCH',
        method_id='securitycenter.projects.updateVirtualMachineThreatDetectionSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta2/{+name}',
        request_field='virtualMachineThreatDetectionSettings',
        request_type_name='SecuritycenterProjectsUpdateVirtualMachineThreatDetectionSettingsRequest',
        response_type_name='VirtualMachineThreatDetectionSettings',
        supports_download=False,
    )

    def UpdateWebSecurityScannerSettings(self, request, global_params=None):
      r"""Update the WebSecurityScannerSettings resource.

      Args:
        request: (SecuritycenterProjectsUpdateWebSecurityScannerSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (WebSecurityScannerSettings) The response message.
      """
      config = self.GetMethodConfig('UpdateWebSecurityScannerSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateWebSecurityScannerSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta2/projects/{projectsId}/webSecurityScannerSettings',
        http_method='PATCH',
        method_id='securitycenter.projects.updateWebSecurityScannerSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta2/{+name}',
        request_field='webSecurityScannerSettings',
        request_type_name='SecuritycenterProjectsUpdateWebSecurityScannerSettingsRequest',
        response_type_name='WebSecurityScannerSettings',
        supports_download=False,
    )



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Code for the gcloud shell help window."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import io

from googlecloudsdk.calliope import cli_tree_markdown as markdown
from googlecloudsdk.command_lib.interactive import parser
from googlecloudsdk.core.document_renderers import render_document
from googlecloudsdk.core.document_renderers import token_renderer
from prompt_toolkit.layout import controls


class HelpWindowControl(controls.UIControl):
  """Implementation of the help window."""

  def __init__(self, default_char=None):
    self._default_char = default_char

  def create_content(self, cli, width, height):
    data = GenerateHelpContent(cli, width)

    return controls.UIContent(
        lambda i: data[i],
        line_count=len(data),
        show_cursor=False,
        default_char=self._default_char)


def GenerateHelpContent(cli, width):
  """Returns help lines for the current token."""
  if width > 80:
    width = 80
  doc = cli.current_buffer.document
  args = cli.parser.ParseCommand(doc.text_before_cursor)
  if not args:
    return []
  arg = args[-1]

  if arg.token_type in (parser.ArgTokenType.GROUP, parser.ArgTokenType.COMMAND):
    return GenerateHelpForCommand(cli, arg, width)
  elif arg.token_type == parser.ArgTokenType.FLAG:
    return GenerateHelpForFlag(cli, arg, width)
  elif arg.token_type == parser.ArgTokenType.FLAG_ARG:
    return GenerateHelpForFlag(cli, args[-2], width)
  elif arg.token_type == parser.ArgTokenType.POSITIONAL:
    return GenerateHelpForPositional(cli, arg, width)

  return []


def GenerateHelpForCommand(cli, token, width):
  """Returns help lines for a command token."""
  lines = []

  # Get description
  height = 4
  gen = markdown.CliTreeMarkdownGenerator(token.tree, cli.root)
  gen.PrintSectionIfExists('DESCRIPTION', disable_header=True)
  doc = gen.Edit()
  fin = io.StringIO(doc)
  lines.extend(render_document.MarkdownRenderer(
      token_renderer.TokenRenderer(
          width=width, height=height), fin=fin).Run())

  lines.append([])  # blank line

  # Get synopis
  height = 5
  gen = markdown.CliTreeMarkdownGenerator(token.tree, cli.root)
  gen.PrintSynopsisSection()
  doc = gen.Edit()
  fin = io.StringIO(doc)
  lines.extend(render_document.MarkdownRenderer(
      token_renderer.TokenRenderer(
          width=width, height=height, compact=False), fin=fin).Run())

  return lines


def GenerateHelpForFlag(cli, token, width):
  """Returns help lines for a flag token."""
  gen = markdown.CliTreeMarkdownGenerator(cli.root, cli.root)
  gen.PrintFlagDefinition(token.tree)
  mark = gen.Edit()

  fin = io.StringIO(mark)
  return render_document.MarkdownRenderer(
      token_renderer.TokenRenderer(
          width=width, height=cli.config.help_lines), fin=fin).Run()


def GenerateHelpForPositional(cli, token, width):
  """Returns help lines for a positional token."""
  gen = markdown.CliTreeMarkdownGenerator(cli.root, cli.root)
  gen.PrintPositionalDefinition(markdown.Positional(token.tree))
  mark = gen.Edit()

  fin = io.StringIO(mark)
  return render_document.MarkdownRenderer(
      token_renderer.TokenRenderer(
          width=width, height=cli.config.help_lines), fin=fin).Run()



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags for commands that deal with the Org Policies API."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.resource_manager import completers
from googlecloudsdk.command_lib.util.args import common_args


def AddIdArgToParser(parser):
  base.Argument(
      'id', metavar='ORG_POLICY_ID',
      help='The Org Policy constraint name.').AddToParser(parser)


def AddParentResourceFlagsToParser(parser):
  """Add flags for the resource ID and enable custom --project flag to be added by modifying the parser.

  Adds --organization, --folder, and --project flags to the parser. The flags
  are added as a required group with a mutex condition, which ensures that the
  user passes in exactly one of the flags.

  Args:
    parser: ArgumentInterceptor, An argparse parser.
  """
  resource_group = parser.add_mutually_exclusive_group(
      required=True,
      help='Resource that is associated with the organization policy.')
  resource_group.add_argument(
      '--organization',
      metavar='ORGANIZATION_ID',
      completer=completers.OrganizationCompleter,
      help='Organization ID.')
  resource_group.add_argument(
      '--folder', metavar='FOLDER_ID', help='Folder ID.')
  common_args.ProjectArgument(
      help_text_to_overwrite='Project ID.').AddToParser(resource_group)



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Create a Workflow."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.cloudbuild.v2 import client_util
from googlecloudsdk.api_lib.cloudbuild.v2 import input_util
from googlecloudsdk.api_lib.cloudbuild.v2 import workflow_input_util
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.cloudbuild import run_flags
from googlecloudsdk.core import log
from googlecloudsdk.core import resources


@base.Hidden
@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class Create(base.CreateCommand):
  """Create a Workflow."""

  @staticmethod
  def Args(parser):
    """Register flags for this command.

    Args:
      parser: An argparse.ArgumentParser-like object. It is mocked out in order
        to capture some information, but behaves like an ArgumentParser.
    """
    parser.add_argument(
        '--file',
        required=True,
        help='The YAML file to use as the Workflow configuration file.')
    run_flags.AddsRegionResourceArg(parser)

  def Run(self, args):
    """This is what gets called when the user runs this command."""
    client = client_util.GetClientInstance()
    messages = client_util.GetMessagesModule()

    yaml_data = input_util.LoadYamlFromPath(args.file)
    workflow = workflow_input_util.CloudBuildYamlDataToWorkflow(yaml_data)

    region_ref = args.CONCEPTS.region.Parse()
    parent = region_ref.RelativeName()
    workflow_id = yaml_data['name']
    name = '%s/workflows/%s' % (parent, workflow_id)

    # Update workflow (or create if missing).
    workflow.name = name
    update_operation = client.projects_locations_workflows.Patch(
        messages.CloudbuildProjectsLocationsWorkflowsPatchRequest(
            name=name, workflow=workflow, allowMissing=True))

    update_operation_ref = resources.REGISTRY.ParseRelativeName(
        update_operation.name,
        collection='cloudbuild.projects.locations.operations')

    updated_workflow = waiter.WaitFor(
        waiter.CloudOperationPoller(client.projects_locations_workflows,
                                    client.projects_locations_operations),
        update_operation_ref, 'Updating Workflow')

    updated_workflow_ref = resources.REGISTRY.Parse(
        updated_workflow.name,
        collection='cloudbuild.projects.locations.workflows',
        api_version=client_util.RELEASE_TRACK_TO_API_VERSION[
            self.ReleaseTrack()],
    )

    log.status.Print('Apply result: {}'.format(updated_workflow_ref))
    return updated_workflow



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Auxiliary data for implementing Service Proxy flags in Instance Templates."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import enum


class TracingState(str, enum.Enum):
  ON = 'ON'
  OFF = 'OFF'


DISCLAIMER = """DISCLAIMER:
This service-proxy configuration format is not a public API and may change
without notice. Please use gcloud command-line tool to run service proxy on
Google Compute Engine."""

# Don't put sudo when running service-proxy-agent-bootstrap.sh, as exported
# variables don't get passed to the script when run with sudo. It's not a
# problem because all commands inside service-proxy-agent-bootstrap.sh
# are run with sudo anyway.
startup_script = """#! /bin/bash
ZONE=$( curl --silent http://metadata.google.internal/computeMetadata/v1/instance/zone -H Metadata-Flavor:Google | cut -d/ -f4 )
export SERVICE_PROXY_AGENT_DIRECTORY=$(mktemp -d)
sudo gsutil cp \
  gs://gce-service-proxy-${ZONE}/service-proxy-agent/releases/service-proxy-agent-0.2.tgz \
  ${SERVICE_PROXY_AGENT_DIRECTORY} \
  || sudo gsutil cp \
    gs://gce-service-proxy/service-proxy-agent/releases/service-proxy-agent-0.2.tgz \
    ${SERVICE_PROXY_AGENT_DIRECTORY}
sudo tar -xzf ${SERVICE_PROXY_AGENT_DIRECTORY}/service-proxy-agent-0.2.tgz -C ${SERVICE_PROXY_AGENT_DIRECTORY}
${SERVICE_PROXY_AGENT_DIRECTORY}/service-proxy-agent/service-proxy-agent-bootstrap.sh"""

startup_script_with_location_template = """#! /bin/bash
export SERVICE_PROXY_AGENT_DIRECTORY=$(mktemp -d)
sudo gsutil cp %s ${SERVICE_PROXY_AGENT_DIRECTORY}
ARCHIVE_NAME=$(ls ${SERVICE_PROXY_AGENT_DIRECTORY})
sudo tar -xzf ${SERVICE_PROXY_AGENT_DIRECTORY}/${ARCHIVE_NAME} -C ${SERVICE_PROXY_AGENT_DIRECTORY}
${SERVICE_PROXY_AGENT_DIRECTORY}/service-proxy-agent/service-proxy-agent-bootstrap.sh"""

startup_script_for_asm_service_proxy = """#! /bin/bash
ISTIOD_ENTRY="{ingress_ip} istiod-{asm_revision}.istio-system.svc"
if ! grep -Fq "${{ISTIOD_ENTRY}}" /etc/hosts; then
  echo "${{ISTIOD_ENTRY}} # Added by Anthos Service Mesh" | sudo tee -a /etc/hosts
fi
DEFAULT_ISTIOD_ENTRY="{ingress_ip} istiod.istio-system.svc"
if ! grep -Fq "${{DEFAULT_ISTIOD_ENTRY}}" /etc/hosts; then
  echo "${{DEFAULT_ISTIOD_ENTRY}} # Added by Anthos Service Mesh" | sudo tee -a /etc/hosts
fi
SERVICE_PROXY_AGENT_BUCKET=$(curl http://metadata.google.internal/computeMetadata/v1/instance/attributes/gce-service-proxy-agent-bucket -H Metadata-Flavor:Google)
ARCHIVE_NAME=$(basename ${{SERVICE_PROXY_AGENT_BUCKET}})
export SERVICE_PROXY_AGENT_DIRECTORY=$(mktemp -d)
sudo gsutil cp ${{SERVICE_PROXY_AGENT_BUCKET}} ${{SERVICE_PROXY_AGENT_DIRECTORY}}
sudo tar -xzf ${{SERVICE_PROXY_AGENT_DIRECTORY}}/${{ARCHIVE_NAME}} -C ${{SERVICE_PROXY_AGENT_DIRECTORY}}
${{SERVICE_PROXY_AGENT_DIRECTORY}}/service-proxy-agent/service-proxy-agent-bootstrap.sh"""

startup_script_for_asm_service_proxy_installer = """#! /bin/bash
INSTALLER_BUCKET=$(curl http://metadata.google.internal/computeMetadata/v1/instance/attributes/gce-service-proxy-installer-bucket -H Metadata-Flavor:Google)
export INSTALLER_DIRECTORY=$(mktemp -d)
sudo gsutil cp ${INSTALLER_BUCKET} ${INSTALLER_DIRECTORY}
INSTALLER_NAME=$(basename ${INSTALLER_BUCKET})
sudo tar -xzf ${INSTALLER_DIRECTORY}/${INSTALLER_NAME} -C ${INSTALLER_DIRECTORY}
sudo ${INSTALLER_DIRECTORY}/installer/installer"""



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for constructing Assured api messages."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.assured import util
from googlecloudsdk.calliope.base import ReleaseTrack


def GetMessages(release_track):
  return util.GetMessagesModule(release_track)


def GetWorkloadMessage(release_track):
  return WORKLOAD_MAP.get(release_track)


def GetKmsSettings(release_track):
  return KMS_SETTINGS_MAP.get(release_track)


def GetResourceSettings(release_track):
  return RESOURCE_SETTINGS_MAP.get(release_track)


def CreateAssuredParent(organization_id, location):
  return 'organizations/{}/locations/{}'.format(organization_id, location)


def CreateAssuredWorkload(display_name=None,
                          compliance_regime=None,
                          billing_account=None,
                          next_rotation_time=None,
                          rotation_period=None,
                          labels=None,
                          etag=None,
                          provisioned_resources_parent=None,
                          resource_settings=None,
                          enable_sovereign_controls=None,
                          release_track=ReleaseTrack.GA):
  """Construct an Assured Workload message for Assured Workloads Beta API requests.

  Args:
    display_name: str, display name of the Assured Workloads environment.
    compliance_regime: str, the compliance regime, which is one of:
      FEDRAMP_MODERATE, FEDRAMP_HIGH, IL4 or CJIS.
    billing_account: str, the billing account of the Assured Workloads
      environment in the form: billingAccounts/{BILLING_ACCOUNT_ID}
    next_rotation_time: str, the next key rotation time for the Assured
      Workloads environment, for example: 2020-12-30T10:15:00.00Z
    rotation_period: str, the time between key rotations, for example: 172800s
    labels: dict, dictionary of label keys and values of the Assured Workloads
      environment.
    etag: str, the etag of the Assured Workloads environment.
    provisioned_resources_parent: str, parent of the provisioned projects,
      for example: folders/{FOLDER_ID}
    resource_settings: list of key=value pairs to set customized resource
      settings, which can be one of the following: consumer-project-id,
        consumer-project-name, encryption-keys-project-id,
      encryption-keys-project-name or keyring-id, for example:
        consumer-project-id={ID1},encryption-keys-project-id={ID2}
    enable_sovereign_controls: bool, whether to enable sovereign controls for
      the Assured Workloads environment.
    release_track: ReleaseTrack, gcloud release track being used

  Returns:
    A populated Assured Workloads message for the Assured Workloads Beta API.
  """

  workload_message = GetWorkloadMessage(release_track)
  workload = workload_message()
  if etag:
    workload.etag = etag
  if billing_account:
    workload.billingAccount = billing_account
  if display_name:
    workload.displayName = display_name
  if labels:
    workload.labels = CreateLabels(labels, workload_message)
  if compliance_regime:
    workload.complianceRegime = workload_message.ComplianceRegimeValueValuesEnum(
        compliance_regime)
  if provisioned_resources_parent:
    workload.provisionedResourcesParent = provisioned_resources_parent
  if next_rotation_time and rotation_period:
    workload.kmsSettings = GetKmsSettings(release_track)(
        nextRotationTime=next_rotation_time, rotationPeriod=rotation_period)
  if resource_settings:
    workload.resourceSettings = CreateResourceSettingsList(
        resource_settings, release_track)
  if enable_sovereign_controls:
    workload.enableSovereignControls = enable_sovereign_controls
  return workload


def CreateAssuredWorkloadsParent(organization_id, location, workload_id):
  return 'organizations/{}/locations/{}/workloads/{}'.format(
      organization_id, location, workload_id)


def CreateLabels(labels, workload_message):
  workload_labels = []
  for key, value in labels.items():
    new_label = workload_message.LabelsValue.AdditionalProperty(
        key=key, value=value)
    workload_labels.append(new_label)
  return workload_message.LabelsValue(additionalProperties=workload_labels)


def CreateResourceSettingsList(resource_settings, release_track):
  """Construct a list of ResourceSettings for Assured Workload object.

  Args:
    resource_settings: a list of key=value pairs of customized resource
      settings.
    release_track: ReleaseTrack, gcloud release track being used.

  Returns:
    A list of ResourceSettings for the Assured Workload object.
  """
  resource_settings_dict = {}
  for key, value in resource_settings.items():
    resource_type = GetResourceType(key, release_track)
    resource_settings = resource_settings_dict[
        resource_type] if resource_type in resource_settings_dict else CreateResourceSettings(
            resource_type, release_track)
    if key.endswith('-id'):
      resource_settings.resourceId = value
    elif key.endswith('-name'):
      resource_settings.displayName = value
    resource_settings_dict[resource_type] = resource_settings
  return list(resource_settings_dict.values())


def GetResourceType(key, release_track):
  """Returns a resource settings type from the key.

  Args:
    key: str, the setting name, which can be one of the following -
      consumer-project-id, consumer-project-name, encryption-keys-project-id,
      encryption-keys-project-name or keyring-id.
    release_track: ReleaseTrack, gcloud release track being used.
  """
  resource_settings_message = GetResourceSettings(release_track)
  if key.startswith('consumer-project'):
    return resource_settings_message.ResourceTypeValueValuesEnum.CONSUMER_PROJECT
  elif key.startswith('encryption-keys-project'):
    return resource_settings_message.ResourceTypeValueValuesEnum.ENCRYPTION_KEYS_PROJECT
  elif key.startswith('keyring'):
    return resource_settings_message.ResourceTypeValueValuesEnum.KEYRING


def CreateResourceSettings(resource_type, release_track):
  resource_settings_message = GetResourceSettings(release_track)
  return resource_settings_message(resourceType=resource_type)


def CreateUpdateMask(display_name, labels):
  update_mask = []
  if display_name:
    update_mask.append('workload.display_name')
  if labels:
    update_mask.append('workload.labels')
  return ','.join(update_mask)


def CreateCreateRequest(external_id,
                        parent,
                        workload,
                        release_track=ReleaseTrack.GA):
  """Construct an Assured Workload Create Request for Assured Workloads API requests.

  Args:
    external_id: str, the identifier that identifies this Assured Workloads
      environment externally.
    parent: str, the parent organization of the Assured Workloads environment
      to be created, in the form: organizations/{ORG_ID}/locations/{LOCATION}.
    workload: Workload, new Assured Workloads environment containing the values
      to be used.
    release_track: ReleaseTrack, gcloud release track being used

  Returns:
    A populated Assured Workloads Update Request for the Assured Workloads API.
  """
  if release_track == ReleaseTrack.GA:
    return util.GetMessagesModule(
        release_track
    ).AssuredworkloadsOrganizationsLocationsWorkloadsCreateRequest(
        externalId=external_id,
        parent=parent,
        googleCloudAssuredworkloadsV1Workload=workload)
  else:
    return util.GetMessagesModule(
        release_track
    ).AssuredworkloadsOrganizationsLocationsWorkloadsCreateRequest(
        externalId=external_id,
        parent=parent,
        googleCloudAssuredworkloadsV1beta1Workload=workload)


def CreateUpdateRequest(workload,
                        name,
                        update_mask,
                        release_track=ReleaseTrack.GA):
  """Construct an Assured Workload Update Request for Assured Workloads API requests.

  Args:
    workload: googleCloudAssuredworkloadsV1beta1Workload, new Assured Workloads
      environment containing the new configuration values to be used.
    name: str, the name for the Assured Workloads environment being updated
      in the form:
        organizations/{ORG_ID}/locations/{LOCATION}/workloads/{WORKLOAD_ID}.
    update_mask: str, list of the fields to be updated, for example,
      workload.display_name,workload.labels
    release_track: ReleaseTrack, gcloud release track being used

  Returns:
    A populated Assured Workloads Update Request for the Assured Workloads API.
  """
  messages = util.GetMessagesModule(release_track)
  if release_track == ReleaseTrack.GA:
    return messages.AssuredworkloadsOrganizationsLocationsWorkloadsPatchRequest(
        googleCloudAssuredworkloadsV1Workload=workload,
        name=name,
        updateMask=update_mask)
  else:
    return messages.AssuredworkloadsOrganizationsLocationsWorkloadsPatchRequest(
        googleCloudAssuredworkloadsV1beta1Workload=workload,
        name=name,
        updateMask=update_mask)


WORKLOAD_MAP = {
    ReleaseTrack.ALPHA:
        GetMessages(ReleaseTrack.ALPHA
                   ).GoogleCloudAssuredworkloadsV1beta1Workload,
    ReleaseTrack.BETA:
        GetMessages(ReleaseTrack.BETA
                   ).GoogleCloudAssuredworkloadsV1beta1Workload,
    ReleaseTrack.GA:
        GetMessages(ReleaseTrack.GA).GoogleCloudAssuredworkloadsV1Workload
}

KMS_SETTINGS_MAP = {
    ReleaseTrack.ALPHA:
        GetMessages(ReleaseTrack.ALPHA
                   ).GoogleCloudAssuredworkloadsV1beta1WorkloadKMSSettings,
    ReleaseTrack.BETA:
        GetMessages(ReleaseTrack.BETA
                   ).GoogleCloudAssuredworkloadsV1beta1WorkloadKMSSettings,
    ReleaseTrack.GA:
        GetMessages(ReleaseTrack.GA
                   ).GoogleCloudAssuredworkloadsV1WorkloadKMSSettings
}

RESOURCE_SETTINGS_MAP = {
    ReleaseTrack.ALPHA:
        GetMessages(ReleaseTrack.ALPHA
                   ).GoogleCloudAssuredworkloadsV1beta1WorkloadResourceSettings,
    ReleaseTrack.BETA:
        GetMessages(ReleaseTrack.BETA
                   ).GoogleCloudAssuredworkloadsV1beta1WorkloadResourceSettings,
    ReleaseTrack.GA:
        GetMessages(ReleaseTrack.GA
                   ).GoogleCloudAssuredworkloadsV1WorkloadResourceSettings
}



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utility for retrieving and parsing the Resource Map."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import abc

from googlecloudsdk.calliope import base as calliope_base
from googlecloudsdk.core import exceptions
from googlecloudsdk.core import yaml
from googlecloudsdk.core import yaml_validator
from googlecloudsdk.core.util import files
import six

_RELEASE_TRACKS = [
    six.text_type(track) for track in calliope_base.ReleaseTrack.AllValues()
]


class ResourceMapError(exceptions.Error):
  """General Purpose Exception."""


class ResourceMapInitializationError(ResourceMapError):
  """Exception for when an error occurs while initializing the resource map."""

  def __init__(self, init_error):
    super(ResourceMapInitializationError,
          self).__init__('Error while '
                         'initializing resource map: [{}]'.format(init_error))


class PrivateAttributeNotFoundError(ResourceMapError):
  """Exception for when a private attribute that doesn't exist is accessed."""

  def __init__(self, data_wrapper, attribute_name):
    super(PrivateAttributeNotFoundError,
          self).__init__('[{}] does not have private attribute [{}].'.format(
              data_wrapper, attribute_name))


class ApiNotFoundError(ResourceMapError):
  """Exception for when an API does not exist in the ResourceMap."""

  def __init__(self, api_name):
    super(ApiNotFoundError,
          self).__init__('[{}] API not found in ResourceMap.'.format(api_name))


class ApiAlreadyExistsError(ResourceMapError):
  """Exception for when an API being added already exists in the ResourceMap."""

  def __init__(self, api_name):
    super(ApiAlreadyExistsError, self).__init__(
        '[{}] API already exists in ResourceMap.'.format(api_name))


class ResourceNotFoundError(ResourceMapError):
  """Exception for when a Resource does not exist in the API."""

  def __init__(self, resource_name):
    super(ResourceNotFoundError, self).__init__(
        '[{}] resource not found in ResourceMap.'.format(resource_name))


class ResourceAlreadyExistsError(ResourceMapError):
  """Exception for when a Resource being added already exists in the ResourceMap."""

  def __init__(self, api_name):
    super(ResourceAlreadyExistsError, self).__init__(
        '[{}] API already exists in ResourceMap.'.format(api_name))


class MetadataNotFoundError(ResourceMapError):
  """Exception for when a metadata field does not exist in the Resource."""

  def __init__(self, resource_name, metadata_field):
    super(MetadataNotFoundError, self).__init__(
        '[{}] metadata field not found in [{}] Resource.'.format(
            metadata_field, resource_name))


class TrackLevelResourceReleaseTrackError(ResourceMapError):
  """Exception for when an attempt to access a releast track of a RT occurs."""

  def __init__(self, attempted_rt, accessed_rt):
    super(TrackLevelResourceReleaseTrackError,
          self).__init__('Attempted accessing of [{}'
                         '] track of TrackLevelResourceData[{}'
                         ']'.format(attempted_rt, accessed_rt))


class MetadataAlreadyExistsError(ResourceMapError):
  """Exception for when a metadata field does not exist in the Resource."""

  def __init__(self, resource_name, metadata_field):
    super(MetadataAlreadyExistsError, self).__init__(
        '[{}] metadata already exists in [{}] Resource.'.format(
            metadata_field, resource_name))


class UnwrappedDataException(ResourceMapError):
  """Exception for when unwrapped data is added to the map."""

  def __init__(self, field_type, data):
    super(UnwrappedDataException, self).__init__(
        'The following data must be wrapped in a(n) {}Data wrapper prior to '
        'being added to the resource map: [{}]'
        .format(field_type, data))


class ResourceMapBase(six.with_metaclass(abc.ABCMeta)):
  """Base data wrapper class for Resource Map metadata yaml files.

  This object loads the relevant resource map file upon instantiation and sets
  the parsed dictionary as the internal attribute _resource_map_data. Underlying
  dictionary data is never interacted with directly, and is instead is
  set/retrieved/interacted with via an ApiData wrapper object.

  Attributes:
    _resource_map_data: Dict containing metadata for each resource in each api.
  """

  def __init__(self):
    self._map_file_path = None
    self._schema_file_path = None
    self._register_paths()
    self._resource_map_data = {}
    self._load_resource_map()

  def __getattr__(self, api_name):
    """Returns underlying API data when accessing attribute."""
    if api_name.startswith('_'):
      raise PrivateAttributeNotFoundError('ResourceMap', api_name)
    return self.get_api(api_name)

  def __contains__(self, api_name):
    """Returns True if api_name exists in self._resource_map_data."""
    return api_name in self._resource_map_data

  def __iter__(self):
    """Yields ApiData wrapper objects for each API in _resource_map_data."""
    for api_name, api_data in six.iteritems(self._resource_map_data):
      yield ApiData(api_name, api_data)

  def __eq__(self, other):
    return self.to_dict() == other.to_dict()

  @abc.abstractmethod
  def _register_paths(self):
    """Must be overridden by child classes to register map and schema paths.

    Must explicitly set self._map_file_path and self._schema_file_path to
    appropriate filepaths in the overridden method of the child class.
    """
    pass

  def _load_resource_map(self):
    """Loads the ~/resource_map.yaml file into self._resource_map_data."""
    try:
      with files.FileReader(self._map_file_path) as f:
        self._resource_map_data = yaml.load(f)
      if not self._resource_map_data:
        self._resource_map_data = {}
    except files.MissingFileError as err:
      raise ResourceMapInitializationError(err)

  def _export_resource_map(self, file_path=None, prune=False, validate=True):
    """Prunes and exports self._resource_map_data to ~/resource_map.yaml."""
    try:
      if prune:
        self.prune()
      if validate:
        self._validate_resource_map()
      with files.FileWriter(file_path or self._map_file_path) as f:
        yaml.dump(self._resource_map_data, stream=f)
    except files.MissingFileError as err:
      raise ResourceMapInitializationError(err)

  def _validate_resource_map(self):
    """Validates resource map against ~/resource_map_schema.yaml."""
    yaml_validator.Validator(self._schema_file_path).Validate(
        self._resource_map_data)

  def to_dict(self):
    return self._resource_map_data

  def prune(self):
    """Prunes the resource map, removing redundant metadata values in the map.

    Calls prune() on each ApiData wrapper object, which in turn calls prune()
    on each underlying resource. Pruning each resource will remove any instances
    of a track-specific metadata field being set to the same value as the parent
    resource metadata field, eliminating any redundancies and keeping the map
    as clean as possible.
    """

    for api_data in iter(self):
      api_data.prune()

  def get_api(self, api_name):
    """Returns the api data wrapped in an ApiData object."""
    if api_name not in self._resource_map_data:
      raise ApiNotFoundError(api_name)
    return ApiData(api_name, self._resource_map_data[api_name])

  def add_api(self, api_data):
    """Adds an api to the resource map.

    Args:
      api_data: Data for api being added. Must be wrapped in an ApiData object.

    Raises:
      ApiAlreadyExistsError: API already exists in resource map.
      UnwrappedDataException: API data attempting to be added without being
        wrapped in an ApiData wrapper object.
    """
    if not isinstance(api_data, ApiData):
      raise UnwrappedDataException('Api', api_data)
    elif api_data.get_api_name() in self._resource_map_data:
      raise ApiAlreadyExistsError(api_data.get_api_name())
    else:
      self._resource_map_data.update(api_data.to_dict())

  def update_api(self, api_data):
    """Updates an API's data with the provided api data.

    Args:
      api_data: API Data to update the api with. Must be provided as an ApiData
      object.

    Raises:
      ApiNotFoundError: Api to be updated does not exist.
      UnwrappedDataException: API data attempting to be added without being
        wrapped in an ApiData wrapper object.
    """
    if not isinstance(api_data, ApiData):
      raise UnwrappedDataException('Api', api_data)
    if api_data.get_api_name() not in self._resource_map_data:
      raise ApiNotFoundError(api_data.get_api_name())
    else:
      self._resource_map_data.update(api_data.to_dict())

  def remove_api(self, api_name):
    """Removes an API from the resource map."""
    if api_name not in self._resource_map_data:
      raise ApiNotFoundError(api_name)
    del self._resource_map_data[api_name]

  def export(self, file_path=None):
    """Public method to export resource map to file."""
    self._export_resource_map(file_path)


class ApiData(object):
  """Data wrapper for an API object in the Resource Map metadata file.

  Attributes:
    _api_name: Name of the API.
    _api_data: Dict of resources and associated metadata constituting the api.
  """

  def __init__(self, api_name, api_data):
    self._api_name = api_name
    self._api_data = api_data

  def __getattr__(self, resource_name):
    """Returns the specified resource's data wrapped in a ResourceData object."""
    if resource_name.startswith('_'):
      raise PrivateAttributeNotFoundError('ApiData', resource_name)
    return ResourceData(resource_name, self._api_name,
                        self._api_data[resource_name])

  def __contains__(self, resource_name):
    return resource_name in self._api_data

  def __iter__(self):
    """Yields ResourceData wrapper objects for each API in _resource_map_data."""
    for resource_name, resource_data in self._api_data.items():
      yield ResourceData(resource_name, self._api_name, resource_data)

  def __repr__(self):
    return repr(self._api_data)

  def __eq__(self, other):
    return self.to_dict() == other.to_dict()

  def to_str(self):
    return six.text_type(self.to_dict())

  def to_dict(self):
    return {self.get_api_name(): self._api_data}

  def get_api_name(self):
    return six.text_type(self._api_name)

  def get_resource(self, resource_name):
    """Returns the data for the specified resource in a ResourceData object."""
    if resource_name not in self._api_data:
      raise ResourceNotFoundError(resource_name)
    return ResourceData(resource_name, self._api_name,
                        self._api_data[resource_name])

  def add_resource(self, resource_data):
    if not isinstance(resource_data, ResourceData):
      raise UnwrappedDataException('Resource', resource_data)
    elif resource_data.get_resource_name() in self._api_data:
      raise ResourceAlreadyExistsError(resource_data.get_resource_name())
    else:
      self._api_data.update(resource_data.to_dict())

  def update_resource(self, resource_data):
    if not isinstance(resource_data, ResourceData):
      raise UnwrappedDataException('Resource', resource_data)
    elif resource_data.get_resource_name() not in self._api_data:
      raise ResourceNotFoundError(resource_data.get_resource_name())
    else:
      self._api_data.update(resource_data.to_dict())

  def remove_resource(self, resource_name, must_exist=True):
    if must_exist and resource_name not in self._api_data:
      raise ResourceNotFoundError(resource_name)
    del self._api_data[resource_name]

  def prune(self):
    for resource_data in iter(self):
      resource_data.prune()


class ResourceData(object):
  """Data wrapper for a Resource object in the ResourceMap metadata file.

  Attributes:
    _resource_name: Name of the resource.
    _api_name: Name of the parent api.
    _resource_data: Metadata for the resource.
  """

  def __init__(self, resource_name, api_name, resource_data):
    self._resource_name = resource_name
    self._api_name = api_name
    self._resource_data = resource_data

  def __getattr__(self, metadata_field):
    """Returns metadata value or TrackLevelResourceData object.

    Attribute being accessed will be either a metadata field for the resource,
    or the release track (GA, BETA, or ALPHA). If the attribute is a metadata
    field the appropriate value will be returned from self._resource_data. If
    the atatribute is a release track, a TrackLevelResourceData object will be
    returned. This enables both of the following usecases:

      `value = map.api.resource.metadata_field` OR
      'value = map.api.resource.ALPHA.metadata_field`

    Args:
      metadata_field: Field or release track being accessed

    Returns:
      Metadata field value OR TrackLevelResourceData object.

    Raises:
      MetadataNotFoundError: Metadata field does not exist.
      PrivateAttributeNotFoundError: Private attribute doesn't exist in object.

    """
    if metadata_field in _RELEASE_TRACKS:
      return self.get_release_track_data(metadata_field)
    elif metadata_field.startswith('_'):
      raise PrivateAttributeNotFoundError('ResourceData', metadata_field)
    else:
      return self.get_metadata(metadata_field)

  def __setattr__(self, metadata_field, value):
    """Sets the specified metadata field to the provided value.

    If the object is not yet instantiated, then standard __setattr__ behavior
    is observed, allowing for proper object instantiation. After initialization,
    the specified metadata field within self._resource_data is set to the
    provided value

    Args:
      metadata_field: Metadata field to set the value for.
      value: Value to set the specified metadata field to.

    Returns:
      True
    """

    if metadata_field.startswith('_'):
      super(ResourceData, self).__setattr__(metadata_field, value)
    elif metadata_field not in self._resource_data:
      self.add_metadata(metadata_field, value)
    else:
      self.update_metadata(metadata_field, value)

  def __eq__(self, other):
    return self.to_dict() == other.to_dict()

  def __contains__(self, metadata_field):
    return self.has_metadata_field(metadata_field)

  def prune(self):
    """Removes any redundant metadata specifications between track and top."""
    for track in _RELEASE_TRACKS:
      if track in self._resource_data:
        track_resource_data = self._resource_data[track]

        for key in list(track_resource_data.keys()):
          if key in self._resource_data and self._resource_data[
              key] == track_resource_data[key]:
            del track_resource_data[key]

        if not track_resource_data:
          del self._resource_data[track]

  def to_dict(self):
    return {self.get_resource_name(): self._resource_data}

  def has_metadata_field(self, metadata_field):
    return metadata_field in self._resource_data

  def get_resource_name(self):
    return self._resource_name

  def get_api_name(self):
    return self._api_name

  def get_full_collection_name(self):
    return '{}.{}'.format(self.get_api_name(), self.get_resource_name())

  def get_metadata(self, metadata_field):
    if metadata_field not in self._resource_data:
      raise MetadataNotFoundError(self._resource_name, metadata_field)
    return self._resource_data[metadata_field]

  def get_release_track_data(self, release_track):
    return TrackLevelResourceData(
        self._resource_name,
        self._api_name,
        self._resource_data,
        track=release_track)

  def add_metadata(self, metadata_field, value):
    if metadata_field in self._resource_data:
      raise MetadataAlreadyExistsError(self._resource_name, metadata_field)
    else:
      self._resource_data[metadata_field] = value

  def update_metadata(self, metadata_field, value):
    if metadata_field not in self._resource_data:
      raise MetadataNotFoundError(self._resource_name, metadata_field)
    else:
      self._resource_data[metadata_field] = value

  def remove_metadata(self, metadata_field):
    if metadata_field not in self._resource_data:
      raise MetadataNotFoundError(self._resource_name, metadata_field)
    else:
      del self._resource_data[metadata_field]


class TrackLevelResourceData(ResourceData):
  """Data wrapper for track-specific resource metadata.

  This data wrapper represents the metadata for a specific release track of a
  resource. Retrieval of metadata will first check for a track level
  specification of the metadata, and if not found will then retrieve the
  top level metadata value.

  Attributes:
    _resource_name: Name of the resource.
    _api_name: Name of the parent api.
    _resource_data: Metadata for the resource.
    _track: Release track for the resource.
    _track_resource_data: Track specific metadata for the resource.
  """

  def __init__(self, resource_name, api_name, resource_data, track):
    self._track = track
    self._track_resource_data = resource_data.get(self._track, {})
    super(TrackLevelResourceData, self).__init__(resource_name, api_name,
                                                 resource_data)

  def __getattr__(self, metadata_field):
    """Retrieves the track-specific metadata value for the resource.

    If the specified release track does not have a specified value, the parent
    metadata field value for the resource will be returned.

    Args:
      metadata_field: Metadata field value to retrieve

    Returns:
      Metadata field value for the specified release track-specific or the
      parent metadata field.

    Raises:
      MetadataNotFoundError: Metadata field value wasn't found for the specific
      track or for the parent.
      PrivateAttributeNotFoundError: Private attribute doesn't exist in object.
    """
    if metadata_field.startswith('_'):
      raise PrivateAttributeNotFoundError('TrackLevelResourceData',
                                          metadata_field)
    else:
      return self.get_metadata(metadata_field)

  def __setattr__(self, metadata_field, value):
    """Sets the specified metadata field to the provided value.

    If the object is not yet instantiated, then standard __setattr__ behavior
    is observed, allowing for proper object intitialization. After
    initialization, the specified metadata field for the release track is set
    to the provided value.

    Args:
      metadata_field: Metadata field to set the value for.
      value: Value to set the specified metadata field to.

    Returns:
      True
    """

    if metadata_field.startswith('_'):
      super(TrackLevelResourceData, self).__setattr__(metadata_field, value)
    else:
      if metadata_field in self._track_resource_data:
        return self.update_metadata(metadata_field, value)
      else:
        return self.add_metadata(metadata_field, value)

  def to_dict(self):
    return {self._resource_name: self._resource_data}

  def get_metadata(self, metadata_field):
    if metadata_field in self._track_resource_data:
      return self._track_resource_data[metadata_field]
    elif metadata_field in self._resource_data:
      return self._resource_data[metadata_field]
    else:
      raise MetadataNotFoundError(self._resource_name, metadata_field)

  def add_metadata(self, metadata_field, value):
    if metadata_field in self._track_resource_data:
      raise MetadataAlreadyExistsError(self._resource_name, metadata_field)
    else:
      self._track_resource_data[metadata_field] = value

  def update_metadata(self, metadata_field, value):
    if metadata_field not in self._track_resource_data:
      raise MetadataNotFoundError(self._resource_name, metadata_field)
    else:
      self._track_resource_data[metadata_field] = value

  def remove_metadata(self, metadata_field):
    if metadata_field not in self._track_resource_data:
      raise MetadataNotFoundError(self._resource_name, metadata_field)
    else:
      del self._track_resource_data[metadata_field]

  def get_release_track(self):
    return self._track

  def get_release_track_data(self, release_track):
    raise TrackLevelResourceReleaseTrackError(release_track, self._track)
